/*
** ocd - ojnk cd player
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <ocd.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <sys/stat.h>

static void volbar(u_char y, u_char x, u_char vol);

extern WINDOW *mwin;
extern int max_x, max_y;
extern u_char sel;

#define DIV1H 25
#define DIV1V 7
#define DIV2V (DIV1V)
#define DIV2H (DIV1H + 2)

#define TL_FIRST (DIV1V + 1)
#define TL_LAST (max_y - 4)
#define TL_LINES ((TL_LAST - TL_FIRST) + 1)

int main_window(struct cdrom *cdrom) {
	u_int track, curpos_y;
	u_char j, *tbuf;

	tbuf = xcalloc(cdrom->cddb.llen + 1, sizeof(u_char));

	/* Status Box */
	wmove(mwin, 0, 0);
	wattron(mwin, A_BOLD | COLOR_PAIR(8));
	whline(mwin, ACS_HLINE, max_x);
	wmove(mwin, 0, 2);
	waddstr(mwin, "[");

	wattrset(mwin, COLOR_PAIR(5));
	waddstr(mwin, "o");

	wattron(mwin, A_BOLD);
	waddstr(mwin, "c");

	wattron(mwin, COLOR_PAIR(7));
	waddstr(mwin, "d");

	wattron(mwin, COLOR_PAIR(8));
	waddstr(mwin, "(");

	wattrset(mwin, COLOR_PAIR(7));
	waddstr(mwin, VERSION);

	wattron(mwin, A_BOLD);
	waddstr(mwin, ".");

	wattroff(mwin, A_BOLD);
	waddstr(mwin, REVISION);

	wattron(mwin, COLOR_PAIR(8) | A_BOLD);
	waddstr(mwin, ")]");

/* ugly hack */
	j = ((max_x - strlen(cdrom->cddb.title)) >> 1) - 1;
	if (j < 14) j = 14;

	wmove(mwin, 0, j);
	wattron(mwin, COLOR_PAIR(8));
	waddstr(mwin, "(");
	wattron(mwin, COLOR_PAIR(7));
	if (cdrom->disc.opened == 1)
		waddstr(mwin, "No Disc");
	else
		waddstr(mwin, cdrom->cddb.title);
	wattron(mwin, COLOR_PAIR(8));
	waddstr(mwin, ")");

	wmove(mwin, DIV1V, 0);
	whline(mwin, ACS_HLINE, max_x);
	wmove(mwin, 1, DIV1H);
	wvline(mwin, ACS_VLINE, DIV2V - 1);

	boldword(mwin, 1, 2, "s", "tatus", ": ", 6);
	boldword(mwin, 2, 4, "m", "ode", ": ", 6);
	boldword(mwin, 3, 2, "v", "olume", ": ", 6);
	boldword(mwin, 4, 3, "t", "rack", ": ", 6);
	boldword(mwin, 5, 4, "t", "ime", ": ", 6);
	boldword(mwin, 6, 6, "C", "D", ": ", 6);

	if (cdrom->disc.opened == 0) {
		ocd_update(cdrom->fd, &cdrom->disc);

		switch (cdrom->disc.status) {
			case CDROM_AUDIO_COMPLETED:
			case CDROM_AUDIO_NO_STATUS:
				wattrset(mwin, COLOR_PAIR(2));
				mvwaddstr(mwin, 1, 11, "Stopped");
				break;
			case CDROM_AUDIO_PLAY:
				wattrset(mwin, A_BOLD | COLOR_PAIR(2));
				mvwaddstr(mwin, 1, 11, "Playing");
				break;
			case CDROM_AUDIO_PAUSED:
				wattrset(mwin, A_BOLD | COLOR_PAIR(3));
				mvwaddstr(mwin, 1, 11, "Paused ");
				break;
			case CDROM_AUDIO_INVALID:
			case CDROM_AUDIO_ERROR:
			default:
				wattrset(mwin, A_BOLD | COLOR_PAIR(1));
				mvwaddstr(mwin, 1, 11, "Error  ");
				break;
		}

		wattrset(mwin, 0);

		switch (cdrom->mode) {
			case REP_T:
				mvwaddstr(mwin, 2, 11, "Repeat   ");
				break;
			case REP_C:
				mvwaddstr(mwin, 2, 11, "Repeat CD");
				break;
			case SHUFFLE:
				mvwaddstr(mwin, 2, 11, "Shuffle  ");
				break;
			case PLAYLIST:
				mvwaddstr(mwin, 2, 11, "Playlist ");
				break;
			case NORMAL:
			default:
				mvwaddstr(mwin, 2, 11, "Normal   ");
				break;
		}
	} else {
		wattrset(mwin, A_BOLD | COLOR_PAIR(1));
		mvwaddstr(mwin, 1, 11, "No Disc");
		mvwaddstr(mwin, 2, 11, "No Disc  ");
		wattrset(mwin, 0);
	}

	mvwprintw(mwin, 3, 11, "%u%%  ",
		ocd_rint(100 * (cdrom->vol.channel0 / 255.0)));
	mvwprintw(mwin, 4, 11, "%u of %u   ",
		cdrom->disc.c_track, cdrom->disc.tracks);

	if (cdrom->disc.status == CDROM_AUDIO_PLAY || cdrom->disc.status == CDROM_AUDIO_PAUSED) {
		mvwprintw(mwin, 5, 11, "%02u:%02u / %02u:%02u",
			cdrom->disc.t_pos.min, cdrom->disc.t_pos.sec,
			cdrom->disc.track[cdrom->disc.c_track - 1].min,
			cdrom->disc.track[cdrom->disc.c_track - 1].sec);
	} else
		mvwaddstr(mwin, 5, 11, "--:-- / --:--");

	mvwprintw(mwin, 6, 11, "%02u:%02u / %02u:%02u",
		cdrom->disc.pos.min, cdrom->disc.pos.sec,
		cdrom->disc.min, cdrom->disc.sec);

/* Command box */
	cmdline(mwin, 1, DIV2H, "+", "next", "E", "eject", "D", "CDDB window");
	cmdline(mwin, 2, DIV2H, "-", "prev", "C", "close tray", "T", "goto time");
	cmdline(mwin, 3, DIV2H, "S", "stop", "G", "goto track", "L", "repeat track");
	cmdline(mwin, 4, DIV2H, "P", "pause", ">", "vol up", "X", "shuffle mode");
	cmdline(mwin, 5, DIV2H, "R", "rewind", "<", "vol down", "O", "playlist");
	cmdline(mwin, 6, DIV2H, "F", "forward", "0", "mute", "Q", "quit");
	wattrset(mwin, 0);

/* Track List */
	curpos_y = DIV1V + 1;
	wmove(mwin, curpos_y, 0);
	wclrtobot(mwin);

	wattron(mwin, A_BOLD | COLOR_PAIR(8));
	wmove(mwin, max_y - 3, 0);
	wvline(mwin, ACS_VLINE, 3);
	whline(mwin, ACS_HLINE, 3);
	wmove(mwin, max_y - 1, 0);
	whline(mwin, ACS_HLINE, 54);
	mvwhline(mwin, max_y - 3, 0, ACS_ULCORNER, 1);
	mvwhline(mwin, max_y - 1, 0, ACS_LLCORNER, 1);
	wattroff(mwin, A_BOLD | COLOR_PAIR(8));

	boldword(mwin, max_y - 3, 3, "v", "olume", "", 6);
	volbar(max_y - 2, 2, cdrom->vol.channel0);

	if (cdrom->disc.opened == 0) {	
		if (sel > TL_LINES)
			track = sel - TL_LINES;
		else
			track = 0;

		for (j = 0 ;j < TL_LINES && track < cdrom->disc.tracks ; track++, j++) {
			if (track + 1 == cdrom->disc.c_track)
				wattrset(mwin, ATTR_CURTRK);
			else if (track == cdrom->disc.c_track)
				wattrset(mwin, 0);
			if (track + 1 == sel)
				wattrset(mwin, ATTR_CURSEL);
			else if (track == sel) {
				if (track + 1 == cdrom->disc.c_track)
					wattrset(mwin, ATTR_CURTRK);
				else
					wattrset(mwin, 0);
			}

			if (cdrom->disc.track[track].data == 1)
				wattron(mwin, A_DIM);
			snprintf(tbuf, cdrom->cddb.llen + 1, "[%+2u:%02u] %02u - %s",
				cdrom->disc.track[track].min, cdrom->disc.track[track].sec,
				track + 1, cdrom->cddb.track[track].title);
			mvwprintw(mwin, curpos_y++, 2, "%-*s", cdrom->cddb.llen, tbuf);
			wattrset(mwin, 0);
		}
	}

	free(tbuf);
	wrefresh(mwin);
	return (0);
}

static void volbar(u_char y, u_char x, u_char vol) {
	u_char blocks, i, percent;

	blocks = vol / 5;
	percent = 100 * blocks / 51; /* 255 / 5 == 51 (max blocks) */

/* XXX fix colors, percents, etc. solid bold cyan for now */
	if (percent < 16)
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else if (percent < 31) 
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else if (percent < 46)
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else if (percent < 61)
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else if (percent < 76)
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else if (percent < 91)
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));
	else
		wattrset(mwin, A_BOLD | COLOR_PAIR(11));

	for (i = 0 ; i < blocks ; i++)
		mvwaddstr(mwin, y, x++, " ");
	wattrset(mwin, 0);
}
