/* cddb.c */
int cddb_lookup(struct cdrom *cd);
u_long cddb_discid(int fd, const disc_t *d);
int cddb_setup(const disc_t *disc, cddb_t *cddb);
int cddb_get(const disc_t *d, const cddb_t *cddb, FILE *fp, const char *server, int port);
int cddb_submit(const cddb_t *cddb, const char *genre, const char *server, int port);
void cddb_clear(cddb_t *cddb);
int cddb_read(FILE *fp, cddb_t *cddb);
int cddb_valid_genre(const char *genre);
int parse_playlist(u_char *p, cddb_t *cddb);
void cddb_unknown(FILE *fp, const disc_t *d, u_long discid);

/* cddb_window.c */
void cddb_control(struct cdrom *cdrom);
void cddb_window(cddb_t *cddb);
int cddb_edit(cddb_t *cddb);
u_long cddb_inexact(int sock, u_char *genre, size_t len);

/* cdrom.c */
struct track *ocd_alloctracks(int fd);
int ocd_setup(const char *device, disc_t *disc);
int ocd_stop(int fd);
int ocd_pause(int fd);
int ocd_resume(int fd);
int ocd_eject(int fd);
int ocd_close(int fd);
int ocd_getvol(int fd, struct cdrom_volctrl *volume);
int ocd_setvol(int fd, const struct cdrom_volctrl *volume);
int ocd_update(int fd, disc_t *disc);
int ocd_cdinfo(int fd, disc_t *disc);
int ocd_get_offsets(int fd, u_char tracks, struct ocd_time *offsets);
int ocd_play_track_pos(int fd, u_char start, u_char end, u_int startf);
int ocd_play_track(int fd, u_char track);
int ocd_play_tracks(int fd, u_char start, u_char end);
int ocd_play_pos(int fd, u_char track, u_int start);
int ocd_play(int fd, u_char track);

/* gen_window.c */
void user_input(WINDOW *this_win, u_char y, u_char x, u_char pad, u_char *gotobuf, size_t len);
void setup_colors(void);

/* main.c */
int main(int argc, char **argv);
int ocd_rint(double orig);

/* main_window.c */
int main_window(struct cdrom *cdrom);

/* util.c */
char *xstrncpy(char *dest, const char *src, size_t n);
char *xstrncat(char *dest, const char *src, size_t n);
void *xcalloc(size_t nmemb, size_t size);
void *xmalloc(size_t size);
int check_dir(const u_char *dir);
int get_addr(const u_char *hostname, u_long *addr);
int host_connect(const u_char *host, int port);
int ocd_getline(FILE *fp, u_char *buf, size_t len);
ssize_t sock_write(int sock, void *buf, size_t len);
ssize_t sock_readline(int fd, u_char *buf, size_t len);
int astrcmp(const u_char *const orig, const u_char *const *const w, size_t n);
void destroy_playlist(struct plist *p);
struct plist *append_node(u_char track, struct plist **p);

